part of 'bodytype_bloc.dart';

abstract class BodytypeState extends Equatable {
  const BodytypeState();

  @override
  List<Object> get props => [];
}

class BodytypeInitial extends BodytypeState {
  const BodytypeInitial();
}

class BodytypeLoading extends BodytypeState {
  const BodytypeLoading();
}

class BodytypeReady extends BodytypeState {
  const BodytypeReady();
}

class BodytypeFinished extends BodytypeState {
  const BodytypeFinished();
}

class BodytypeError extends BodytypeState {
  final String error;
  const BodytypeError({required this.error});

  @override
  List<Object> get props => [error];
}
